import logging
import time
import docker

import longvideo_config
import rov_server_config
import utils
from longvideo_update import server_health_check


def server_restart(slb_client, instance_id, image_name):
    try:
        logging.info(f"instance：{instance_id}")
        # 1. 获取ip
        ip_address = utils.get_ip_address(client=slb_client, instance_id=instance_id)
        logging.info(f"ip address：{instance_id}/{ip_address}")
        # 2. 摘流量
        utils.set_instance_weight_process_with_slbs(client=slb_client,
                                          slb_id=longvideo_config.slb_id_list,
                                          instance_id_list=[instance_id],
                                          weight_list=[(0, 30)])

        logging.info(f"set weight with 0 finished.")
        # 3. 重启容器
        docker_client = docker.DockerClient(base_url=f'tcp://{ip_address}:2375', timeout=60)
        image_id = docker_client.containers.get(image_name)
        image_id.restart()
        logging.info(f"docker restart finished.")
        # 4. 探活
        server_health_check(slb_client, instance_id)
        time.sleep(30)
        logging.info(f"health check finished.")
        # 5. 挂流量
        add_weight_list = [(10, 10), (20, 10), (40, 10), (60, 10), (80, 10), (100, 10)]
        utils.set_instance_weight_process(client=slb_client,
                                          slb_id=rov_server_config.slb_id,
                                          instance_id_list=[instance_id],
                                          weight_list=add_weight_list)
        logging.info(f"server restart finished, instance: {instance_id}/{ip_address}")
    except Exception as e:
        logging.error(f"server restart fail, instance: {instance_id}")
        logging.error(e)


def main():
    try:
        slb_client = utils.connect_client(access_key_id=longvideo_config.slb_client_params['access_key_id'],
                                          access_key_secret=longvideo_config.slb_client_params['access_key_secret'],
                                          region_id=longvideo_config.slb_client_params['region_id'])

        # 1. 获取slb下所有机器
        online_instance_ids = utils.get_instance_ids(client=slb_client, slb_id=longvideo_config.slb_id_list[0])

        online_instance_ids = online_instance_ids[:1]

        online_instance_count = len(online_instance_ids)
        logging.info(f"online instance count: {online_instance_count}.")
        logging.info(f"online instance ids: {online_instance_ids}")


        # 2. 逐台重启
        image_name = 'longvideoapi'
        for i, instance_id in enumerate(online_instance_ids):
            server_restart(slb_client=slb_client, instance_id=instance_id, image_name=image_name)
            logging.info(f"重启进度: {i+1}/{online_instance_count}")
        logging.info(f"server restart finished!")
    except Exception as e:
        logging.error(e)


if __name__ == '__main__':
    main()